"use strict";
/**
 *  @author Prashant Nayak
 *  @copyright ©2015 Intellify Learning.  All Rights Reserved.
 *  @license For license information contact, info@intellifylearning.org
 */

/**
 *
 * Entities are used to describe objects that are part of an Event
 *
 * Note that you would not typically use a generic Entity, but rather a specific type (e.g. VideoObject)

 *  Here is an example:
 * <pre><code>
 * var quiz = new Assessment("https://some-university.edu/politicalScience/2014/american-revolution-101/assessment1");
 * quiz.setName("American Revolution - Key Figures Assessment");
 * quiz.setIsPartOf("https://some-university.edu/politicalScience/2014/american-revolution-101");
 * quiz.setDateModified(1402965614516);
 * quiz.setDateCreated(1402965614516);
 * quiz.setDatePublished(1402965614516);
 * quiz.setDateToActivate(1402965614516);
 * quiz.setDateToShow(1402965614516);
 * quiz.setDateToStartOn(1402965614516);
 * quiz.setDateToSubmit(1402965614516);
 * quiz.setMaxAttempts(2);
 * quiz.setMaxSubmits(2);
 * quiz.setMaxScore(3.0);
 *  @constructor
 */
function Entity() {
  // Constructor
}

/**
 * Set the event id
 *  Note that this manifests as a JSON-LD @id
 *
 * @param {string} id - id URI
 */
Entity.prototype.setId = function(id) {
  this['@id'] = id;
};

/**
 * Set the event type
 *  Note that this manifests as a JSON-LD @type
 *
 * @param {string} type - id type
 */
Entity.prototype.setType = function(type) {
  this['@type'] = type;
};

/**
 * Set the event name
 *
 * @param {string} name - name
 */
Entity.prototype.setName = function(name) {
  this.name = name;
};

/**
 * Set the event description
 *
 * @param {string} description - description
 */
Entity.prototype.setDescription = function(description) {
  this.description = description;
};

/**
 * Set addtional (custom) properties
 *
 * @param {map} properties - properties
 */
Entity.prototype.setProperties = function(properties) {
  this.properties = properties;
};

/**
 * Set the time the entity was created
 *  A ISO-8601 date
 *
 * @param {string} dateCreated - string representing datetime
 */
Entity.prototype.setDateCreated = function(dateCreated) {
  this.dateCreated = dateCreated;
};

/**
 * Set the time the entity was last modified
 *  A ISO-8601 date
 *
 * @param {string} dateModified - string representing datetime
 */
Entity.prototype.setDateModified = function(dateModified) {
  this.dateModified = dateModified;
};

/**
 * Enum for JSON-LD types of entities
 * @readonly
 * @enum {string}
 */
Entity.prototype.Types = {
  "ACTIVITY_CONTEXT": "http://purl.imsglobal.org/caliper/v1/ActivityContext",
  "AGENT": "http://purl.imsglobal.org/caliper/v1/Agent",
  "ATTEMPT": "http://purl.imsglobal.org/caliper/v1/Attempt",
  "DIGITAL_RESOURCE": "http://purl.imsglobal.org/caliper/v1/DigitalResource",
  "ENTITY": "http://purl.imsglobal.org/caliper/v1/Entity",
  "LEARNING_OBJECTIVE": "http://purl.imsglobal.org/caliper/v1/LearningObjective",
  "RESULT": "http://purl.imsglobal.org/caliper/v1/Result",
  "VIEW": "http://purl.imsglobal.org/caliper/v1/View",
  "SESSION": "http://purl.imsglobal.org/caliper/v1/Session"
};

module.exports = Entity;
