"use strict";
/**
 *  @author Prashant Nayak
 *  @copyright ©2015 Intellify Learning.  All Rights Reserved.
 *  @license For license information contact, info@intellifylearning.org
 */

/**
 * 
 * Events are used to describe the interaction between an Agent (user or system) with a object (learning activity).
 * 
 *  As such, events are constructed with a triple - actor-action-object
 * 
 *  In addition, interactions might also generate other objects (e.g. a student interacting with a Quiz will
 *  typically generate an Attempt).  
 * 
 *  Also, the actor might interact with an object to target another (e.g. a student might login to an edApp, 
 *  with a target being a Quiz).  
 *  
 *  As a result, most events typically either specify a target or a generated object in addition to the actor-action-object triple
 *  
 *  Events also can specify a startedAtTime and optionally an endedAtTime and a duration
 * 
 *  Note that you would not typically use a generic Event, but rather a specific type (e.g. AssessmentEvent)
 *
 *  Here is an example:
 * <pre><code>
 * var assessmentEvent = new Event();
 * assessmentEvent.setActor(actor);
 * assessmentEvent.setAction(action);
 * assessmentEvent.setObject(quiz);
 * assessmentEvent.setGenerated(attempt);
 * assessmentEvent.setEdApp(edApp);
 * assessmentEvent.setLisOrganization(institution);
 * assessmentEvent.setStartedAtTime(1402965614516);
 * </pre></code>
 *  @constructor
 */
function Event() {
  // Constructor
}

/**
 * Set the event context
 * Note that this manifests as a JSON-LD @context. When you create one of the standard event types, the context and type are automatically set for you.
 * E.g. in the code below, the navigationEvent object has its context and type pre-set when you use the factory constructor
 *  new Intellify.Events.NavigationEvent()
 * <pre><code>
 * var actor = new Intellify.Entities.Person("https://some-university.edu/user/554433");
 * var navigationEvent = new Intellify.Events.NavigationEvent();
 * navigationEvent.setActor(actor);
 * </code></pre>
 *
 * @param {string} context - context URI
 */
Event.prototype.setContext = function(context) {
  this['@context'] = context;
};

/**
 * Set the event type
 * Note that this manifests as a JSON-LD @type
 *
 * @param {string} type - type URI
 */
Event.prototype.setType = function(type) {
  this['@type'] = type;
};

/**
 * Set the event actor
 *
 * E.g.
 * <pre><code>
 * var actor = new Intellify.Entities.Person("https://some-university.edu/user/554433");
 * var navigationEvent = new Intellify.Events.NavigationEvent();
 * navigationEvent.setActor(actor);
 * </code></pre>
 * @param {Intellify.Entities.Agent} actor - agent
 */
Event.prototype.setActor = function(actor) {
  this.actor = actor;
};

/**
 * Set the event action
 * Use the profile under Intellify.Actions
 *
 * E.g.
 * <pre><code>
 * // The Action for the Learning Event
 * var action = Intellify.Actions.ReadingActions.NAVIGATED_TO;
 * var navigationEvent = new Intellify.Events.NavigationEvent();
 * navigationEvent.setAction(action);
 * </code></pre>
 *
 * @param {string} actor - agent
 */
Event.prototype.setAction = function(action) {
  this.action = action;
};

/**
 * Set the event actor
 *
 * @param {Intellify.Entities.Agent} actor - agent
 */
Event.prototype.setObject = function(object) {
  this.object = object;
};

/**
 * Set the event target. 
 *  Target types are specified by each Metric Profile
 *
 * @param {object} target - object representing target
 */
Event.prototype.setTarget = function(target) {
  this.target = target;
};

/**
 * Set the object generated by the event. 
 *  Generated types are specified by each Metric Profile
 *
 * @param {object} generated - object representing generated
 */
Event.prototype.setGenerated = function(generated) {
  this.generated = generated;
};

/**
 * Set the time the event started at. 
 *  A ISO-8601 date
 *
 * @param {string} startedAtTime - string representing datetime
 */
Event.prototype.setStartedAtTime = function(startedAt) {
  this.startedAtTime = startedAt;
};

/**
 * Set the time the event ended at
 *  A ISO-8601 date
 *
 * @param {string} startedAtTime - string representing datetime
 */
Event.prototype.setEndedAtTime = function(endedAt) {
  this.endedAtTime = endedAt;
};

/**
 * Set the duration of the event
 *  A ISO-8601 duration
 *
 * @param {string} duration - string representing duration
 */
Event.prototype.setDuration = function(duration) {
  this.duration = duration;
};

/**
 * Set the organization that the event occured in
 *  This is part of the "learning context" for the event. Organization is typically a course or class
 *
 * @param {Intellify.Entities.CourseSection} lisOrganization 
 * @param {Intellify.Entities.Organization} lisOrganization
 */
Event.prototype.setLisOrganization = function(lisOrganization) {
  this.group = lisOrganization;
};

/**
 * Set the edApp that the event occured in
 *  This is part of the "learning context" for the event. edApp is typically a Software Application
 *
 * @param {Intellify.Entities.SoftwareApplication} edApp
 */
Event.prototype.setEdApp = function(edApp) {
  this.edApp = edApp;
};

/**
 * Enum for JSON-LD contexts for events
 * @readonly
 * @enum {string}
 */
Event.prototype.Contexts = {
  "ANNOTATION": "http://purl.imsglobal.org/ctx/caliper/v1/AnnotationEvent",
  "ASSESSMENT": "http://purl.imsglobal.org/ctx/caliper/v1/AssessmentEvent",
  "ASSESSMENT_ITEM": "http://purl.imsglobal.org/ctx/caliper/v1/AssessmentItemEvent",
  "ASSIGNABLE": "http://purl.imsglobal.org/ctx/caliper/v1/AssignableEvent",
  "EVENT": "http://purl.imsglobal.org/ctx/caliper/v1/Event",
  "MEDIA": "http://purl.imsglobal.org/ctx/caliper/v1/MediaEvent",
  "NAVIGATION": "http://purl.imsglobal.org/ctx/caliper/v1/NavigationEvent",
  "OUTCOME": "http://purl.imsglobal.org/ctx/caliper/v1/OutcomeEvent",
  "VIEWED": "http://purl.imsglobal.org/ctx/caliper/v1/ViewEvent",
  "SESSION": "http://purl.imsglobal.org/ctx/caliper/v1/SessionEvent"
};

/**
 * Enum for JSON-LD types of events
 * @readonly
 * @enum {string}
 */
Event.prototype.Types = {
  "ANNOTATION": "http://purl.imsglobal.org/caliper/v1/AnnotationEvent",
  "ASSESSMENT": "http://purl.imsglobal.org/caliper/v1/AssessmentEvent",
  "ASSESSMENT_ITEM": "http://purl.imsglobal.org/caliper/v1/AssessmentItemEvent",
  "ASSIGNABLE": "http://purl.imsglobal.org/caliper/v1/AssignableEvent",
  "EVENT": "http://purl.imsglobal.org/caliper/v1/Event",
  "MEDIA": "http://purl.imsglobal.org/caliper/v1/MediaEvent",
  "NAVIGATION": "http://purl.imsglobal.org/caliper/v1/NavigationEvent",
  "OUTCOME": "http://purl.imsglobal.org/caliper/v1/OutcomeEvent",
  "VIEWED": "http://purl.imsglobal.org/caliper/v1/ViewEvent",
  "SESSION": "http://purl.imsglobal.org/caliper/v1/SessionEvent"
};

module.exports = Event;
