"use strict";
/**
 *  @author Prashant Nayak
 *  @copyright ©2015 Intellify Learning.  All Rights Reserved.
 *  @license For license information contact, info@intellifylearning.org
 */

var _ = require('lodash-node');
var client = require('./asyncClient');
var logger = require('./logger');

// Grab an existing namespace object
// or create a blank object if it doesn't exist
// so we can attach non-sensor module exports to it
var Intellify = {};
if (typeof window !== "undefined") {
  if (!_.isUndefined(window.Intellify)) {
    Intellify = window.Intellify;
  }
}

// The sensor itself
var IntelliSense = {};

/**
 * Initializes the default client to use. Uses the socket consumer by default.
 * @param  array $options passed straight to the client
 */
IntelliSense.initialize = function(options) {
  if (!_.isUndefined(options)) {
    client.initialize(options);
  }
};


/**
 * Send learning events
 * @deprecated since version 1.0.9. Use the 'send' method instead.
 * @param  CaliperEvent $caliperEvent The Learning Event
 * @return boolean                   whether the measure call succeeded
 */
IntelliSense.measure = function(caliperEvent) {
  client.measure(caliperEvent);
};

/**
 * Send learning events
 * @param  CaliperEvent $caliperEvent The Learning Event
 * @return boolean                   whether the send call succeeded
 */
IntelliSense.send = function(caliperEvent) {
  client.measure(caliperEvent);
};

/**
 * Describe an entity
 * @param  CaliperEntity $caliperEntity The Intellify Entity we are describing
 * @return boolean            whether the describe call succeeded
 */
IntelliSense.describe = function(caliperEntity) {
  client.describe(caliperEntity);
};

/**
 * Login and return authentication token
 * @param  User $user The user (name and password) to login
 * @param long $expiryInSeconds Token expiration duration in seconds
 * @param Function $successCallback Success callback function
 * @param Function $errorCallback Error callback function
 */
IntelliSense.authenticate = function(user, expiryInSeconds, successCallback,
  errorCallback) {
  var successFunction = function(token) {
    successCallback(token);
  };

  var errorFunction = function(message) {
    logger.log("Error in authentication : " + message);
    errorCallback(message);
  };
  client.authenticate(user, expiryInSeconds, successFunction, errorFunction);
};

/**
 * refresh authentication token
 * The call can be made only for authenticated users.
 * @param  String $token user authorization token
 * @param long $expiryInSeconds Token expiration duration in seconds
 * @param Function $successCallback Success callback function
 * @param Function $errorCallback Error callback function
 */
IntelliSense.refreshToken = function(token, expiryInSeconds, successCallback,
  errorCallback) {
  var successFunction = function(token) {
    successCallback(token);
  };

  var errorFunction = function(message) {
    logger.log("Error in refresh token : " + message);
    errorCallback(message);
  };
  client.refreshToken(token, expiryInSeconds, successFunction, errorFunction);
};

/**
 * Query an Intellify stream and return the result.
 * The call can be made only for authenticated users.
 * Hence, we need to call authenticate before placing any query
 * @param String $token user authorization token
 * @param String $stream Name of the stream to be searched
 * @param boolean $isDynamic True if this is dynamic query
 * @param String $urlParams URL parameters if any
 * @param String $requestBody JSON query string which will be the request body
 * @param Function $successCallback Success callback function
 * @param Function $errorCallback Error callback function
 */
IntelliSense.query = function(token, stream, isDynamic, urlParams, requestBody,
  successCallback, errorCallback) {
  var successFunction = function(data) {
    successCallback(data);
  };

  var errorFunction = function(message) {
    logger.log("Error while querying : " + message);
    errorCallback(message);
  };
  client.query(token, stream, isDynamic, urlParams, requestBody, successFunction,
    errorFunction);
};

// Stick on the modules that need to be exported under the Intellify namespace
// You only need to require the top-level modules. Browserify
// will walk the dependency graph and load everything correctly
Intellify.Actions = {};
Intellify.Entities = {};
Intellify.Events = {};

// ACTIONS
Intellify.Actions.AnnotationActions = require('./actions/annotationActions');
Intellify.Actions.AssessmentActions = require('./actions/assessmentActions');
Intellify.Actions.AssessmentItemActions = require(
  './actions/assessmentItemActions');
Intellify.Actions.AssignableActions = require('./actions/assignableActions');
Intellify.Actions.MediaActions = require('./actions/mediaActions');
Intellify.Actions.OutcomeActions = require('./actions/outcomeActions');
Intellify.Actions.ReadingActions = require('./actions/readingActions');
Intellify.Actions.SessionActions = require('./actions/sessionActions');


// ENTITIES
Intellify.Entities.Entity = require('./entities/caliperEntity');

// Core entities
Intellify.Entities.Agent = require('./entities/agent');
Intellify.Entities.DigitalResource = require('./entities/digitalResource');
Intellify.Entities.LearningObjective = require('./entities/learningObjective');
Intellify.Entities.SoftwareApplication = require(
  './entities/softwareApplication');
Intellify.Entities.WebPage = require('./entities/webPage');

//User
Intellify.Entities.User = require('./entities/user');

// LIS entities

Intellify.Entities.Person = require('./entities/lis/person');

Intellify.Entities.CourseSection = require('./entities/lis/courseSection');

Intellify.Entities.Organization = require('./entities/lis/organization');

// Annotation entities
Intellify.Entities.Annotation = require('./entities/annotation/annotation');
Intellify.Entities.BookmarkAnnotation = require(
  './entities/annotation/bookmarkAnnotation');
Intellify.Entities.HighlightAnnotation = require(
  './entities/annotation/highlightAnnotation');
Intellify.Entities.SharedAnnotation = require(
  './entities/annotation/sharedAnnotation');
Intellify.Entities.TagAnnotation = require(
  './entities/annotation/tagAnnotation');

// Assignable and Assessement entities
Intellify.Entities.AssignableDigitalResource = require(
  './entities/assignable/assignableDigitalResource');
Intellify.Entities.Attempt = require('./entities/assignable/attempt');
Intellify.Entities.Assessment = require('./entities/assessment/assessment');
Intellify.Entities.AssessmentItem = require(
  './entities/assessment/assessmentItem');

// Media Entities
Intellify.Entities.MediaObject = require('./entities/media/mediaObject');
Intellify.Entities.VideoObject = require('./entities/media/videoObject');
Intellify.Entities.AudioObject = require('./entities/media/audioObject');
Intellify.Entities.ImageObject = require('./entities/media/imageObject');
Intellify.Entities.MediaLocation = require('./entities/media/mediaLocation');

// Reading Entities
Intellify.Entities.EPubVolume = require('./entities/reading/ePubVolume');
Intellify.Entities.Frame = require('./entities/reading/frame');

// Session Entities
Intellify.Entities.Session = require('./entities/session/session');

// Outcome Entities
Intellify.Entities.Result = require('./entities/outcome/result');

// EVENTS
Intellify.Events.Event = require('./events/caliperEvent');
Intellify.Events.AnnotationEvent = require('./events/annotationEvent');
Intellify.Events.AssessmentEvent = require('./events/assessmentEvent');
Intellify.Events.AssessmentItemEvent = require('./events/assessmentItemEvent');
Intellify.Events.AssignableEvent = require('./events/assignableEvent');
Intellify.Events.MediaEvent = require('./events/mediaEvent');
Intellify.Events.NavigationEvent = require('./events/navigationEvent');
Intellify.Events.OutcomeEvent = require('./events/outcomeEvent');
Intellify.Events.SessionEvent = require('./events/sessionEvent');
Intellify.Events.ViewEvent = require('./events/viewEvent');

// Replace/Create the global namespace and objects (the sensor) we want there
Intellify.Sensor = IntelliSense;

// Replace/create Intellify in global namespace
var win = (typeof window !== "undefined") ? window : undefined;
if (!_.isUndefined(win)) {
  win.Intellify = Intellify;
  logger.log("Added sensor to window global %o", window.Intellify);
  logger.info("Browser mode: intellisense-js available to window global as Intellify");
} else {
  logger.info("Browser mode: intellisense-js NOT made available to window global as Intellify");
}

//logger.log('info', "CommonJS mode: intellisense-js exported as Intellify");
module.exports = Intellify;
